//
// Oliver Wilson <oliver.wilson@ed.ac.uk>
//

#ifndef BITARRAY_H
#define BITARRAY_H

#include <stdint.h>
#include <fstream>

namespace LDHT {

class BitArray {

private:
    bool loaded_;
    unsigned char *bit_array_;
    uint64_t length_;
    bool validArrayIndex(uint64_t);

public:
    BitArray();
    BitArray(uint64_t);
    BitArray(std::istream&);
    BitArray(uint8_t*);
    BitArray(const BitArray&);
    virtual ~BitArray();

    virtual bool fromStream(std::istream&);
    virtual void toStream(std::ostream&);
    virtual void toBuffer(uint8_t*);
    virtual uint64_t getSize();
    virtual uint64_t getLength();
    virtual bool get(uint64_t);
    virtual void set(uint64_t, bool);
    virtual void set(uint64_t);
    virtual void unset(uint64_t);
    virtual bool isSet(uint64_t);
    virtual bool isUnset(uint64_t);
    virtual bool equals(BitArray&);
    virtual uint32_t getBits(uint64_t, uint8_t);
    virtual void setBits(uint64_t, uint8_t, uint32_t);
    #if 0
    uint32_t getBits(uint64_t array_index, uint8_t n_bits) {
        uint64_t mask = (1 << n_bits) - 1;
        int byte_index = array_index / 8;
        int bit_index = array_index % 8;
        mask = mask << bit_index;
        uint64_t array_data = *(uint64_t*)(bit_array_ + byte_index);
        uint32_t value = array_data & mask;
        value = value >> bit_index;
        return value;
    }
    void setBits(uint64_t array_index, uint8_t n_bits, uint32_t bits) {
        uint64_t mask = (1 << n_bits) - 1;
        int byte_index = array_index / 8;
        int bit_index = array_index % 8;
        uint64_t array_data = *(uint64_t*)(bit_array_ + byte_index);
        bits = bits & mask;
        mask = mask << bit_index;
        mask = ~mask;
        array_data = array_data & mask;
        bits = bits << bit_index;
        array_data = array_data | bits;
        *(uint64_t*)(bit_array_ + byte_index) = array_data;
    }
    #endif

    virtual void dump();

};

}  // namespace LDHT.

#endif // BITARRAY_H

