//
// Oliver Wilson <oliver.wilson@ed.ac.uk>
//

#ifndef BLOOMFILTER_H
#define BLOOMFILTER_H

#define MAX_K 100

#include <fstream>
#include <vector>

#include "BitArray.h"

namespace LDHT {

class BloomFilter {

private:

public:
    BitArray* ba_;
    uint64_t m_, k_, P_;
    std::vector< std::vector<uint64_t> > rands;

    BloomFilter(uint64_t, uint64_t);
    BloomFilter(std::ifstream&);
    BloomFilter(const char*);
    BloomFilter(unsigned char*);
    BloomFilter(BloomFilter&);
    ~BloomFilter();
    void toStream(std::ofstream&);
    void toBuffer(unsigned char*);
    uint64_t getSize();
    uint64_t getP();
    uint64_t calck(uint64_t, uint64_t);
    uint64_t getk();
    uint64_t getm();
    BitArray* getBitArray();
    std::vector< uint64_t > getRands(uint64_t);
    uint64_t getRand(uint64_t, uint64_t);
    void add(unsigned char *);
    void add(uint64_t);
    bool contains(unsigned char *);
    bool contains(const char*);
    bool contains(uint64_t);
    uint64_t hash(uint64_t, uint64_t, unsigned char *);
    static uint64_t greaterPrime(uint64_t);
    static bool isPrime(uint64_t);
    bool equals(BloomFilter&);

};

}  // namespace LDHT.

#endif // BLOOMFILTER_H

