//
// Oliver Wilson <oliver.wilson@ed.ac.uk>
//

#ifndef METHODREGISTRY_H
#define METHODREGISTRY_H

#include <map>

#include "Util.h"

namespace LDHT {

#define DEFINE_STATIC_METHOD_REGISTRY(parent_class, name, type) \
    MethodRegistry<type>* parent_class::k_ ## name ## _method_registry = NULL; \
    MethodRegistry<type>* parent_class::get ## name ## Registry() { \
        if (k_ ## name ## _method_registry == NULL) { \
            k_ ## name ## _method_registry = new MethodRegistry<type>(); \
        } \
        return k_ ## name  ## _method_registry; \
    }

#define DECLARE_STATIC_METHOD_REGISTRY(name, type) \
    static MethodRegistry<type>* k_ ## name ## _method_registry; \
    static MethodRegistry<type>* get## name ## Registry() 


#define REGISTER_STATIC_METHOD(parent_class, registry_name, name, method) \
    struct StaticMethodRegistration { \
        StaticMethodRegistration() { \
            parent_class::get ## registry_name ##Registry()->registerMethod(name, method); \
        }; \
    }; \
    static StaticMethodRegistration k_static##nameMethodRegistration

template <class Method>
class MethodRegistry {
public:
    void registerMethod(const char* name, Method method) {
        std::string str_name = name;
        registerMethod(str_name, method);
    }
    void registerMethod(std::string& name, Method method) {
        if (m_method_map.count(name) != 0) {
            V("method with name " << name << " already registered");
            abort();
        }
        else {
            m_method_map[name] = method;
        }
    };
    Method getMethod(const char* name) {
        std::string str_name = name;
        return getMethod(str_name);
    }
    Method getMethod(const std::string& name) {
        typename MethodMap::iterator method_iter = m_method_map.find(name);
        if (method_iter == m_method_map.end()) {
            V("no registered method with name " << name);
            V("did you put REGISTER_METHOD() in your .cpp?");
            abort();
        }
        return (*method_iter).second;
    };
    bool isRegistered(const char* name) {
        std::string str_name = name;
        return isRegistered(str_name);
    }
    bool isRegistered(std::string& name) {
        return (m_method_map.count(name) > 0);
    }
    MethodRegistry* clone() {
        MethodRegistry<Method>* instance = new MethodRegistry<Method>();
        instance->m_method_map.insert(m_method_map.begin(), m_method_map.end());
        return instance;
    }

    typedef std::map<std::string, Method> MethodMap;
    MethodMap m_method_map;
};

}  // namespace LDHT.

#endif  // METHODREGISTRY_H

