//
// Oliver Wilson <oliver.wilson@ed.ac.uk>
//

#ifndef NEWNGRAM_H
#define NEWNGRAM_H

#define MAX_NGRAM_ORDER 5

#include <algorithm>
#include <string>

#include <stdint.h>

#include "TypeDef.h"

namespace LDHT {

class NewNgram {
public:
    friend std::ostream& operator<<(std::ostream& stream,
                                    NewNgram& ngram) {
        stream << ngram.m_order << ", ";
        stream << static_cast<int>(ngram.m_gram_hashes[0]);
        for (int i = 1; i < k_max_order; ++i) {
            stream << ", " << static_cast<int>(ngram.m_gram_hashes[i]);
        }
        return stream;
    } 

    static const char* k_unknown_word;
    static const uint64_t k_unknown_word_hash;
    static const int k_max_order;
    static const uint64_t k_final_hash_a[MAX_NGRAM_ORDER];
    static const uint64_t k_final_hash_b[MAX_NGRAM_ORDER];
    static const uint64_t k_final_hash_P;
    static uint64_t hashGram(const char* gram_str);
    static bool hashGram(const char* gram_str,
                         const char** next_gram_ret,
                         uint64_t* hash_ret);
    static uint64_t hashFinal(const char* gram_str, int idx);

    NewNgram();
    explicit NewNgram(const char*);

    virtual int getOrder() const;
    virtual uint64_t genFingerprint(int start_idx, int end_idx);
    virtual uint64_t genFullFingerprint();
    virtual void appendGram(const char* gram);
    virtual void appendGram(uint64_t hash);
    virtual int compare(const NewNgram& other) const;
    virtual int compareMoses(const NewNgram& other) const;
    virtual void copyFrom(const NewNgram& other);

    int m_order;
    uint64_t m_gram_hashes[MAX_NGRAM_ORDER];

};

}  // namespace LDHT.

#endif  // NEWNGRAM_H

