//
// Oliver Wilson <oliver.wilson@ed.ac.uk>
//

#ifndef TABLECHUNKFACTORY_H
#define TABLECHUNKFACTORY_H

#include "MethodRegistry.h"

namespace LDHT {

class TableChunk;

// TableChunkFactory creates instances of table chunks based on the
// type name supplied.
//
// The default factory uses a static map of name -> factory method which
// subclasses of TableChunk have registered a factory method with.
class TableChunkFactory {
public:
    typedef TableChunk* (*FactoryMethod)();
    DECLARE_STATIC_METHOD_REGISTRY(DefaultMethods, FactoryMethod);
    static TableChunkFactory* createDefaultFactory();

    TableChunkFactory();
    virtual ~TableChunkFactory();

    virtual void setMethodRegistry(
            MethodRegistry<FactoryMethod>* method_registry);
    virtual TableChunk* createInstance(std::string& type);
    virtual TableChunk* createInstance(const char* type);

private:
    MethodRegistry<FactoryMethod>* m_method_registry;

};

}  // namespace LDHT.

#endif  // TABLECHUNKFACTORY_H

