//
// Oliver Wilson <oliver.wilson@ed.ac.uk>
//

#ifndef TABLECHUNKLOSSYDICT_H
#define TABLECHUNKLOSSYDICT_H

#include "TableChunk.h"
#include "ticpp.h"

namespace LDHT {

class BitArray;
class FactoryCollection;
class HashFunction;

class TableChunkLossyDict : public TableChunk {
public:
    static TableChunk* createInstance();

    TableChunkLossyDict();
    virtual ~TableChunkLossyDict();

    virtual void fromXml(FactoryCollection& factory_collection,
                         ticpp::Element& xml_element,
                         int chunk_index);
    virtual bool writeOut();

    std::string getName();
    void setName(std::string& name);

    int getChunkIndex();
    void setChunkIndex(int chunk_index);

    int getNumCells();
    void setNumCells(int num_cells);

    int getNumChunks();
    void setNumChunks(int num_chunks);

    int getNumErrorBits();
    void setNumErrorBits(int num_error_bits);

    int getNumValueBits();
    void setNumValueBits(int num_value_bits);

    int getNumBitsPerCell();
    void recalculateNumBitsPerCell();

    int getNumCellsPerChunk();
    void recalculateNumCellsPerChunk();

    int getLinearProbingLimit();
    void setLinearProbingLimit(int linear_probing_limit);

    uint64_t getErrorMask();
    void recalculateErrorMask();

    uint64_t getIndexMask();
    void recalculateIndexMask();

    BitArray* getBitArray();
    void setBitArray(BitArray* bit_array);

    HashFunction* getLocatorHash();
    void setLocatorHash(HashFunction* locator_hash);

    HashFunction* getErrorHash();
    void setErrorHash(HashFunction* error_hash);

    virtual TableProtocol::OpFlags get(uint64_t tag,
                                       uint64_t key,
                                       uint64_t* value_ret);
    virtual TableProtocol::OpFlags set(uint64_t tag,
                                       uint64_t key,    
                                       uint64_t value);
    virtual TableProtocol::OpFlags increment(uint64_t tag,
                                             uint64_t key,
                                             uint64_t increment);

private:
    std::string m_name;
    std::string m_chunk_dir;
    int m_chunk_index;
    int m_num_cells;
    int m_num_chunks;
    int m_num_value_bits;
    int m_num_error_bits;
    int m_num_bits_per_cell;
    int m_num_cells_per_chunk;
    int m_linear_probing_limit;
    uint64_t m_error_mask;
    uint64_t m_index_mask;
    BitArray* m_bit_array;
    HashFunction* m_locator_hash;
    HashFunction* m_error_hash;
    pthread_rwlock_t m_rwlock;

};

}  // namespace LDHT.

#endif  // TABLECHUNKLOSSYDICT_H

