//
// Oliver Wilson <oliver.wilson@ed.ac.uk>
//

#ifndef TABLEPROTOCOLLOCALADAPTER_H
#define TABLEPROTOCOLLOCALADAPTER_H

#include "TableProtocol.h"

namespace LDHT {

// TableProtocolLocalAdapter acts as glue between any two
// TableProtocolCallbacks. For example it can be used to host a non
// networked instance of a server, which can used by a client.
//
// In this case, the callee would be an instance of the Server, and
// the caller would be an instance of the Client.
//
// Methods called on this class simply forward the calls to the callee,
// and the corresponding response handler is called on the caller.
class TableProtocolLocalAdapter : public TableProtocol {
public:
    TableProtocolLocalAdapter();
    virtual ~TableProtocolLocalAdapter();

    virtual void nop(TableIndex table_index, uint64_t tag);
    virtual void get(TableIndex table_index, uint64_t tag, uint64_t key);
    virtual void set(TableIndex table_index,
                     uint64_t tag,
                     uint64_t key,
                     uint64_t value);
    virtual void increment(TableIndex table_index,
                           uint64_t tag,
                           uint64_t key,
                           uint64_t increment);
    virtual void incrementApprox(TableIndex table_index,
                                 uint64_t tag,
                                 uint64_t key,
                                 uint64_t increment);
    virtual void sync(TableIndex table_index, uint64_t tag);
    virtual void write(TableIndex table_index, uint64_t tag);

    virtual int getWaitFD();

    void setCallee(TableProtocolCallback* callee);
    void setCaller(TableProtocolCallback* caller);

private:
    TableProtocolCallback* m_caller;
    TableProtocolCallback* m_callee;
};

}  // namespace LDHT.

#endif  // TABLEPROTOCOLLOCALADAPTER_H


