//
// Oliver Wilson <oliver.wilson@ed.ac.uk>
//

#ifndef UTIL_H
#define UTIL_H

#include <iostream>

#include <cstdlib>
#include <cstring>
#include <stdint.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

#define V(expr) std::cerr << __FILE__ << ":" \
                          << __LINE__ << " " \
                          << __func__ << ": " \
                          << expr << std::endl

#define VV(varname) V(#varname << ": " << varname);

namespace LDHT {

class DOMElement;

class Util {
public:
    struct CStringEqualityComparer {
        bool operator()(const char* s1, const char* s2) const {
            return (s1 == s2) || (s1 && s2 && strncmp(s1, s2, 1024) == 0);
        }
    };
    struct CStringOrderer {
        bool operator()(const char* s1, const char* s2) const {
            return strncmp(s1, s2, 1024);
        }
    };

    static int createListeningSocket(int port);
    static bool canRead(int);
    static bool canWrite(int);
    static int readyBytes(int fd);
    static uint64_t hashStr(uint64_t, uint64_t, uint64_t, const char*);
    static uint64_t hash(uint64_t a, uint64_t b, uint64_t P, uint8_t* e);
    static uint64_t hash(uint64_t a, uint64_t b, uint64_t P, uint64_t e);
    static inline uint64_t rdtsc() {
#if defined(__powerpc__)
        return __builtin_readcyclecounter();
#else
        uint32_t lo, hi;
        __asm__ __volatile__(
                "xorl %%eax,%%eax\n"
                "cpuid\n"
                "rdtsc\n"
                : "=a" (lo), "=d" (hi)
                :
                : "%ebx", "%ecx");
        return static_cast<uint64_t>(hi) << 32 | lo;
#endif
    };
};

}  // namespace LDHT.

#endif  // UTIL_H

