//
// Oliver Wilson <oliver.wilson@ed.ac.uk>
//

#ifndef VARINTSTREAM_H
#define VARINTSTREAM_H

#include <stdint.h>
#include <cstdlib>

#define RD_BUFFER_SIZE 16384
#define WR_BUFFER_SIZE 16384

namespace LDHT {

class Transport;

class BufferWriteCallback {
public:
    BufferWriteCallback() : m_pending(false) {
    };
    virtual ~BufferWriteCallback() {};

    virtual void preBufferWrite() = 0;
    virtual void postBufferWrite() = 0;

    inline bool isPending() {
        return m_pending;
    };
    inline void setPending() {
        m_pending = true;
    };
    inline void setNotPending() {
        m_pending = false;
    };

private:
    bool m_pending;
};

class VarIntStream {
public:
    static void encode(uint64_t, uint8_t*, uint8_t**);
    static void encodeForceWidth(uint64_t, size_t, uint8_t*, uint8_t**);
    static uint64_t decode(uint8_t*, uint8_t**);
    static size_t optimalWidth(uint64_t n);

    VarIntStream();
    explicit VarIntStream(Transport*);
    ~VarIntStream();

    void initialiseBuffers();

    virtual uint64_t next();
    virtual void put(uint64_t);
    virtual void putForceWidth(uint64_t, size_t);
    virtual uint8_t* reserve(size_t);
    virtual void flush();

    virtual void readFromTransport();
    virtual void writeToTransport();
    virtual bool finishedRdBuffer();
    virtual bool finishedWrBuffer();
    virtual bool couldWrite(size_t num_bytes);

    void setBufferWriteCallback(BufferWriteCallback*);
    Transport* getTransport();
    virtual int getWaitFD();

    int getWriteCounter();
    void incrWriteCounter();
    void resetWriteCounter();

private:
    Transport* m_transport;

    uint8_t* m_rd_buffer;
    size_t m_rd_buffer_size;
    uint8_t* m_rd_buffer_rd_ptr;
    uint8_t* m_rd_buffer_wr_ptr;
    size_t m_rd_buffer_bytes_written;

    uint8_t* m_wr_buffer;
    size_t m_wr_buffer_size;
    uint8_t* m_wr_buffer_rd_ptr;
    uint8_t* m_wr_buffer_wr_ptr;
    size_t m_wr_buffer_bytes_written;

    BufferWriteCallback* m_buffer_write_callback;

    int m_write_counter;
};

}  // namespace LDHT.

#endif  // VARINTSTREAM_H

